@@ -864,6 +864,41 @@ guint64  bd_fs_get_size (const gchar *device, const gchar *fstype, GError **erro
 guint64  bd_fs_get_free_space (const gchar *device, const gchar *fstype, GError **error);
 
 
+/**
+ * bd_fs_get_min_size:
+ * @device: the device with file system to get minimum size for
+ * @fstype: (nullable): the filesystem type on @device or %NULL to detect
+ * @error: (out) (optional): place to store error (if any)
+ *
+ * Get minimum size for filesystem on @device. This calls other fs info functions from this
+ * plugin based on detected filesystem (e.g. bd_fs_ext4_get_min_size for ext4). This
+ * function will return an error for unknown/unsupported filesystems.
+ *
+ * Returns: minimum size of filesystem on @device, 0 in case of error.
+ *
+ * Tech category: %BD_FS_TECH_GENERIC-%BD_FS_TECH_MODE_RESIZE
+ */
+guint64  bd_fs_get_min_size (const gchar *device, const gchar *fstype, GError **error);
+
+
+/**
+ * bd_fs_can_get_info:
+ * @type: the filesystem type to be tested for info querying support
+ * @required_utility: (out) (transfer full): the utility binary which is required
+ *                                           for info querying (if missing i.e. return FALSE but no error)
+ * @error: (out) (optional): place to store error (if any)
+ *
+ * Searches for the required utility to get info of the given filesystem and
+ * returns whether it is installed.
+ * Unknown filesystems or filesystems which do not support info querying result in errors.
+ *
+ * Returns: whether getting filesystem info is available
+ *
+ * Tech category: %BD_FS_TECH_GENERIC-%BD_FS_TECH_MODE_QUERY
+ */
+gboolean  bd_fs_can_get_info (const gchar *type, gchar **required_utility, GError **error);
+
+
 typedef enum {
     BD_FS_MKFS_LABEL     = 1 << 0,
     BD_FS_MKFS_UUID      = 1 << 1,
@@ -1030,6 +1065,24 @@ gboolean  bd_fs_can_get_size (const gchar *type, gchar **required_utility, GErro
 gboolean  bd_fs_can_get_free_space (const gchar *type, gchar **required_utility, GError **error);
 
 
+/**
+ * bd_fs_can_get_min_size:
+ * @type: the filesystem type to be tested for installed minimum size querying support
+ * @required_utility: (out) (transfer full): the utility binary which is required
+ *                                           for size querying (if missing i.e. return FALSE but no error)
+ * @error: (out) (optional): place to store error (if any)
+ *
+ * Searches for the required utility to get minimum size of the given filesystem and
+ * returns whether it is installed.
+ * Unknown filesystems or filesystems which do not support minimum size querying result in errors.
+ *
+ * Returns: whether getting filesystem size is available
+ *
+ * Tech category: %BD_FS_TECH_GENERIC-%BD_FS_TECH_MODE_QUERY
+ */
+gboolean  bd_fs_can_get_min_size (const gchar *type, gchar **required_utility, GError **error);
+
+
 /**
  * bd_fs_mkfs:
  * @device: the device to create the new filesystem on
@@ -1445,6 +1498,45 @@ gboolean  bd_fs_ext3_resize (const gchar *device, guint64 new_size, const BDExtr
 gboolean  bd_fs_ext4_resize (const gchar *device, guint64 new_size, const BDExtraArg **extra, GError **error);
 
 
+/**
+ * bd_fs_ext2_get_min_size:
+ * @device: the device containing the file system to get min size for
+ * @error: (out) (optional): place to store error (if any)
+ *
+ * Returns: smallest shrunken filesystem size as reported by ntfsresize
+ *          in case of error 0 is returned and @error is set
+ *
+ * Tech category: %BD_FS_TECH_EXT2-%BD_FS_TECH_MODE_RESIZE
+ */
+guint64  bd_fs_ext2_get_min_size (const gchar *device, GError **error);
+
+
+/**
+ * bd_fs_ext3_get_min_size:
+ * @device: the device containing the file system to get min size for
+ * @error: (out) (optional): place to store error (if any)
+ *
+ * Returns: smallest shrunken filesystem size as reported by ntfsresize
+ *          in case of error 0 is returned and @error is set
+ *
+ * Tech category: %BD_FS_TECH_EXT3-%BD_FS_TECH_MODE_RESIZE
+ */
+guint64  bd_fs_ext3_get_min_size (const gchar *device, GError **error);
+
+
+/**
+ * bd_fs_ext4_get_min_size:
+ * @device: the device containing the file system to get min size for
+ * @error: (out) (optional): place to store error (if any)
+ *
+ * Returns: smallest shrunken filesystem size as reported by ntfsresize
+ *          in case of error 0 is returned and @error is set
+ *
+ * Tech category: %BD_FS_TECH_EXT4-%BD_FS_TECH_MODE_RESIZE
+ */
+guint64  bd_fs_ext4_get_min_size (const gchar *device, GError **error);
+
+
 /**
  * bd_fs_xfs_mkfs:
  * @device: the device to create a new xfs fs on
@@ -1818,6 +1910,19 @@ gboolean  bd_fs_ntfs_resize (const gchar *device, guint64 new_size, GError **err
 BDFSNtfsInfo* bd_fs_ntfs_get_info (const gchar *device, GError **error);
 
 
+/**
+ * bd_fs_ntfs_get_min_size:
+ * @device: the device containing the file system to get min size for
+ * @error: (out) (optional): place to store error (if any)
+ *
+ * Returns: smallest shrunken filesystem size as reported by ntfsresize
+ *          in case of error 0 is returned and @error is set
+ *
+ * Tech category: %BD_FS_TECH_NTFS-%BD_FS_TECH_MODE_RESIZE
+ */
+guint64  bd_fs_ntfs_get_min_size (const gchar *device, GError **error);
+
+
 /**
  * bd_fs_f2fs_mkfs:
  * @device: the device to create a new f2fs fs on
@@ -1891,6 +1996,19 @@ BDFSF2FSInfo* bd_fs_f2fs_get_info (const gchar *device, GError **error);
 gboolean  bd_fs_f2fs_resize (const gchar *device, guint64 new_size, gboolean safe, const BDExtraArg **extra, GError **error);
 
 
+/**
+ * bd_fs_f2fs_check_label:
+ * @label: label to check
+ * @error: (out) (optional): place to store error
+ *
+ * Returns: whether @label is a valid label for the f2fs file system or not
+ *          (reason is provided in @error)
+ *
+ * Tech category: always available
+ */
+gboolean  bd_fs_f2fs_check_label (const gchar *label, GError **error);
+
+
 /**
  * bd_fs_nilfs2_mkfs:
  * @device: the device to create a new nilfs fs on
