@@ -101,7 +101,7 @@ GType  bd_part_disk_spec_get_type () {
 }
 
 static gboolean  bd_part_is_tech_avail_stub (BDPartTech tech G_GNUC_UNUSED, guint64 mode G_GNUC_UNUSED, GError **error) {
-    g_critical ("The function 'bd_part_is_tech_avail' called, but not implemented!");
+    bd_utils_log_format (BD_UTILS_LOG_CRIT, "The function 'bd_part_is_tech_avail' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_part_is_tech_avail' called, but not implemented!");
     return FALSE;
@@ -124,7 +124,7 @@ gboolean  bd_part_is_tech_avail (BDPartTech tech, guint64 mode, GError **error)
 
 
 static gboolean  bd_part_create_table_stub (const gchar *disk G_GNUC_UNUSED, BDPartTableType type G_GNUC_UNUSED, gboolean ignore_existing G_GNUC_UNUSED, GError **error) {
-    g_critical ("The function 'bd_part_create_table' called, but not implemented!");
+    bd_utils_log_format (BD_UTILS_LOG_CRIT, "The function 'bd_part_create_table' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_part_create_table' called, but not implemented!");
     return FALSE;
@@ -150,7 +150,7 @@ gboolean  bd_part_create_table (const gchar *disk, BDPartTableType type, gboolea
 
 
 static BDPartSpec* bd_part_get_part_spec_stub (const gchar *disk G_GNUC_UNUSED, const gchar *part G_GNUC_UNUSED, GError **error) {
-    g_critical ("The function 'bd_part_get_part_spec' called, but not implemented!");
+    bd_utils_log_format (BD_UTILS_LOG_CRIT, "The function 'bd_part_get_part_spec' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_part_get_part_spec' called, but not implemented!");
     return NULL;
@@ -174,7 +174,7 @@ BDPartSpec* bd_part_get_part_spec (const gchar *disk, const gchar *part, GError
 
 
 static BDPartSpec* bd_part_get_part_by_pos_stub (const gchar *disk G_GNUC_UNUSED, guint64 position G_GNUC_UNUSED, GError **error) {
-    g_critical ("The function 'bd_part_get_part_by_pos' called, but not implemented!");
+    bd_utils_log_format (BD_UTILS_LOG_CRIT, "The function 'bd_part_get_part_by_pos' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_part_get_part_by_pos' called, but not implemented!");
     return NULL;
@@ -199,7 +199,7 @@ BDPartSpec* bd_part_get_part_by_pos (const gchar *disk, guint64 position, GError
 
 
 static BDPartDiskSpec* bd_part_get_disk_spec_stub (const gchar *disk G_GNUC_UNUSED, GError **error) {
-    g_critical ("The function 'bd_part_get_disk_spec' called, but not implemented!");
+    bd_utils_log_format (BD_UTILS_LOG_CRIT, "The function 'bd_part_get_disk_spec' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_part_get_disk_spec' called, but not implemented!");
     return NULL;
@@ -222,7 +222,7 @@ BDPartDiskSpec* bd_part_get_disk_spec (const gchar *disk, GError **error) {
 
 
 static BDPartSpec** bd_part_get_disk_parts_stub (const gchar *disk G_GNUC_UNUSED, GError **error) {
-    g_critical ("The function 'bd_part_get_disk_parts' called, but not implemented!");
+    bd_utils_log_format (BD_UTILS_LOG_CRIT, "The function 'bd_part_get_disk_parts' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_part_get_disk_parts' called, but not implemented!");
     return NULL;
@@ -245,7 +245,7 @@ BDPartSpec** bd_part_get_disk_parts (const gchar *disk, GError **error) {
 
 
 static BDPartSpec** bd_part_get_disk_free_regions_stub (const gchar *disk G_GNUC_UNUSED, GError **error) {
-    g_critical ("The function 'bd_part_get_disk_free_regions' called, but not implemented!");
+    bd_utils_log_format (BD_UTILS_LOG_CRIT, "The function 'bd_part_get_disk_free_regions' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_part_get_disk_free_regions' called, but not implemented!");
     return NULL;
@@ -268,7 +268,7 @@ BDPartSpec** bd_part_get_disk_free_regions (const gchar *disk, GError **error) {
 
 
 static BDPartSpec* bd_part_get_best_free_region_stub (const gchar *disk G_GNUC_UNUSED, BDPartType type G_GNUC_UNUSED, guint64 size G_GNUC_UNUSED, GError **error) {
-    g_critical ("The function 'bd_part_get_best_free_region' called, but not implemented!");
+    bd_utils_log_format (BD_UTILS_LOG_CRIT, "The function 'bd_part_get_best_free_region' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_part_get_best_free_region' called, but not implemented!");
     return NULL;
@@ -300,7 +300,7 @@ BDPartSpec* bd_part_get_best_free_region (const gchar *disk, BDPartType type, gu
 
 
 static BDPartSpec* bd_part_create_part_stub (const gchar *disk G_GNUC_UNUSED, BDPartTypeReq type G_GNUC_UNUSED, guint64 start G_GNUC_UNUSED, guint64 size G_GNUC_UNUSED, BDPartAlign align G_GNUC_UNUSED, GError **error) {
-    g_critical ("The function 'bd_part_create_part' called, but not implemented!");
+    bd_utils_log_format (BD_UTILS_LOG_CRIT, "The function 'bd_part_create_part' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_part_create_part' called, but not implemented!");
     return NULL;
@@ -332,7 +332,7 @@ BDPartSpec* bd_part_create_part (const gchar *disk, BDPartTypeReq type, guint64
 
 
 static gboolean  bd_part_delete_part_stub (const gchar *disk G_GNUC_UNUSED, const gchar *part G_GNUC_UNUSED, GError **error) {
-    g_critical ("The function 'bd_part_delete_part' called, but not implemented!");
+    bd_utils_log_format (BD_UTILS_LOG_CRIT, "The function 'bd_part_delete_part' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_part_delete_part' called, but not implemented!");
     return FALSE;
@@ -356,7 +356,7 @@ gboolean  bd_part_delete_part (const gchar *disk, const gchar *part, GError **er
 
 
 static gboolean  bd_part_resize_part_stub (const gchar *disk G_GNUC_UNUSED, const gchar *part G_GNUC_UNUSED, guint64 size G_GNUC_UNUSED, BDPartAlign align G_GNUC_UNUSED, GError **error) {
-    g_critical ("The function 'bd_part_resize_part' called, but not implemented!");
+    bd_utils_log_format (BD_UTILS_LOG_CRIT, "The function 'bd_part_resize_part' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_part_resize_part' called, but not implemented!");
     return FALSE;
@@ -384,7 +384,7 @@ gboolean  bd_part_resize_part (const gchar *disk, const gchar *part, guint64 siz
 
 
 static gboolean  bd_part_set_part_name_stub (const gchar *disk G_GNUC_UNUSED, const gchar *part G_GNUC_UNUSED, const gchar *name G_GNUC_UNUSED, GError **error) {
-    g_critical ("The function 'bd_part_set_part_name' called, but not implemented!");
+    bd_utils_log_format (BD_UTILS_LOG_CRIT, "The function 'bd_part_set_part_name' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_part_set_part_name' called, but not implemented!");
     return FALSE;
@@ -409,7 +409,7 @@ gboolean  bd_part_set_part_name (const gchar *disk, const gchar *part, const gch
 
 
 static gboolean  bd_part_set_part_type_stub (const gchar *disk G_GNUC_UNUSED, const gchar *part G_GNUC_UNUSED, const gchar *type_guid G_GNUC_UNUSED, GError **error) {
-    g_critical ("The function 'bd_part_set_part_type' called, but not implemented!");
+    bd_utils_log_format (BD_UTILS_LOG_CRIT, "The function 'bd_part_set_part_type' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_part_set_part_type' called, but not implemented!");
     return FALSE;
@@ -434,7 +434,7 @@ gboolean  bd_part_set_part_type (const gchar *disk, const gchar *part, const gch
 
 
 static gboolean  bd_part_set_part_id_stub (const gchar *disk G_GNUC_UNUSED, const gchar *part G_GNUC_UNUSED, const gchar *part_id G_GNUC_UNUSED, GError **error) {
-    g_critical ("The function 'bd_part_set_part_id' called, but not implemented!");
+    bd_utils_log_format (BD_UTILS_LOG_CRIT, "The function 'bd_part_set_part_id' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_part_set_part_id' called, but not implemented!");
     return FALSE;
@@ -459,7 +459,7 @@ gboolean  bd_part_set_part_id (const gchar *disk, const gchar *part, const gchar
 
 
 static gboolean  bd_part_set_part_uuid_stub (const gchar *disk G_GNUC_UNUSED, const gchar *part G_GNUC_UNUSED, const gchar *uuid G_GNUC_UNUSED, GError **error) {
-    g_critical ("The function 'bd_part_set_part_uuid' called, but not implemented!");
+    bd_utils_log_format (BD_UTILS_LOG_CRIT, "The function 'bd_part_set_part_uuid' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_part_set_part_uuid' called, but not implemented!");
     return FALSE;
@@ -484,7 +484,7 @@ gboolean  bd_part_set_part_uuid (const gchar *disk, const gchar *part, const gch
 
 
 static gboolean  bd_part_set_part_bootable_stub (const gchar *disk G_GNUC_UNUSED, const gchar *part G_GNUC_UNUSED, gboolean bootable G_GNUC_UNUSED, GError **error) {
-    g_critical ("The function 'bd_part_set_part_bootable' called, but not implemented!");
+    bd_utils_log_format (BD_UTILS_LOG_CRIT, "The function 'bd_part_set_part_bootable' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_part_set_part_bootable' called, but not implemented!");
     return FALSE;
@@ -509,7 +509,7 @@ gboolean  bd_part_set_part_bootable (const gchar *disk, const gchar *part, gbool
 
 
 static gboolean  bd_part_set_part_attributes_stub (const gchar *disk G_GNUC_UNUSED, const gchar *part G_GNUC_UNUSED, guint64 attrs G_GNUC_UNUSED, GError **error) {
-    g_critical ("The function 'bd_part_set_part_attributes' called, but not implemented!");
+    bd_utils_log_format (BD_UTILS_LOG_CRIT, "The function 'bd_part_set_part_attributes' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_part_set_part_attributes' called, but not implemented!");
     return FALSE;
@@ -534,7 +534,7 @@ gboolean  bd_part_set_part_attributes (const gchar *disk, const gchar *part, gui
 
 
 static const gchar* bd_part_get_part_table_type_str_stub (BDPartTableType type G_GNUC_UNUSED, GError **error) {
-    g_critical ("The function 'bd_part_get_part_table_type_str' called, but not implemented!");
+    bd_utils_log_format (BD_UTILS_LOG_CRIT, "The function 'bd_part_get_part_table_type_str' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_part_get_part_table_type_str' called, but not implemented!");
     return NULL;
@@ -557,7 +557,7 @@ const gchar* bd_part_get_part_table_type_str (BDPartTableType type, GError **err
 
 
 static const gchar* bd_part_get_type_str_stub (BDPartType type G_GNUC_UNUSED, GError **error) {
-    g_critical ("The function 'bd_part_get_type_str' called, but not implemented!");
+    bd_utils_log_format (BD_UTILS_LOG_CRIT, "The function 'bd_part_get_type_str' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_part_get_type_str' called, but not implemented!");
     return NULL;
