@@ -4,8 +4,11 @@ import time
 import overrides_hack
 
 from utils import create_sparse_tempfile, TestTags, tag_test
-from gi.repository import BlockDev, GLib
 
+import gi
+gi.require_version('GLib', '2.0')
+gi.require_version('BlockDev', '3.0')
+from gi.repository import GLib, BlockDev
 
 class LoopTestCase(unittest.TestCase):
 
